using System;
using System.Collections.Generic;
using System.Linq;
using MacGruber;
using MVR.FileManagementSecure;
using SimpleJSON;

namespace CheesyFX
{
    public class PresetSystem
    {
        private MVRScript script;
        private JSONStorableUrl loadURL;
        public string saveDir = "";
        public Func<JSONClass> Store;
        public Action<JSONClass> Load;
        private JSONClass factoryDefaults;

        public PresetSystem(MVRScript script)
        {
            this.script = script;
            loadURL = new JSONStorableUrl("loadURL", "", UILoadJSON, "json", true);
            loadURL.hideExtension = true;
            loadURL.setCallbackFunction -= UILoadJSON;
            loadURL.allowFullComputerBrowse = false;
            loadURL.allowBrowseAboveSuggestedPath = true;
            
        }

        public void Init(bool doLoad = true)
        {
            FileManagerSecure.CreateDirectory(saveDir);
            loadURL.SetFilePath(saveDir);
            loadURL.setCallbackFunction += UILoadJSON;
            factoryDefaults = Store();
            if(doLoad) ApplyLatestMatchingPreset();
        }

        public void CreateUI()
        {
            var twinButton = Utils.SetupTwinButton(script,
                "Save preset", UISaveJSONDialog,
                "Load preset", delegate {}, false);
            loadURL.RegisterFileBrowseButton(twinButton.buttonRight);
            
            Utils.SetupTwinButton(script,
                "UserDefaults", () => ApplyFromUrl(saveDir+"UserDefaults.json"),
                "FactoryDefaults", () => Load(factoryDefaults), true);
        }
        
        private void ApplyFromUrl(string path)
        {
            if(!FileManagerSecure.FileExists(path)) return;
            path = FileManagerSecure.NormalizePath(path);
            {
                JSONClass jc = SuperController.singleton.LoadJSON(path).AsObject;
                Load(jc);
            }
        }

        public void ApplyLatestMatchingPreset()
        {
            List<string> matchingPresetPaths = new List<string>();
            string atomName = script.containingAtom.name;
            string[] paths = FileManagerSecure.GetFiles(saveDir);
            for (int i = 0; i < paths.Length; i++)
            {
                string path = paths[i];
                if (!path.EndsWith(".json")) continue;
                string presetName = path.Replace(saveDir+"\\","");
                
                if(presetName.StartsWith(atomName)) matchingPresetPaths.Add(presetName);
                else
                {
                    if(script.containingAtom.GetComponentInChildren<DAZCharacterSelector>().gender == DAZCharacterSelector.Gender.Male && presetName.StartsWith("UserDefaultsMale")) matchingPresetPaths.Insert(0, presetName);
                    else if (presetName.StartsWith("UserDefaults")) matchingPresetPaths.Insert(0, presetName);
                }
            }

            if (matchingPresetPaths.Count > 0)
            {
                string preset = matchingPresetPaths.Last();
                ApplyFromUrl(saveDir+preset);
                $"Successfully applied '{saveDir+preset}'.".Print();
            }
        }
        
        public void UILoadJSON(string url){
            JSONClass jc = SuperController.singleton.LoadJSON(url).AsObject;
            if(jc != null) Load(jc);
        }
        
        public void UISaveJSONDialog(){
            SuperController.singleton.GetMediaPathDialog(UISaveJSON, "json", saveDir, false, true, false, null, true, null, false, false);
            SuperController.singleton.mediaFileBrowserUI.SetTextEntry(true);
            if(SuperController.singleton.mediaFileBrowserUI.fileEntryField != null){
                string filename = "UserDefaults";
                SuperController.singleton.mediaFileBrowserUI.fileEntryField.text = filename;
                SuperController.singleton.mediaFileBrowserUI.ActivateFileNameField();
            }
        }
        
        public void UISaveJSON(string path)
        {
            if (string.IsNullOrEmpty(path)) return;
            string path1 = path.Replace('\\', '/')+".json";
            JSONClass jc = Store();
            SuperController.singleton.SaveJSON(jc, path1);
            SuperController.singleton.DoSaveScreenshot(path+".jpg");
        }
    }

    // public class MyMVRScript : MVRScript
    // {
    //     public virtual JSONClass Store()
    //     {
    //         return new JSONClass();
    //     }
    //     
    //     public virtual void Load(JSONClass jc){}
    // }
}